<?php

namespace Drupal\viewsreference_extras;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\views\ViewExecutable;
use Drupal\viewsreference\ViewsReferenceCompressionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the views reference extras compression service.
 *
 * This service compresses the entity details then uses those to reload the
 * views reference configuration on un-compress.
 */
class ViewsReferenceExtrasCompressionReload implements ViewsReferenceCompressionInterface, ContainerInjectionInterface {

  /**
   * Constructs the Views Reference Extras compression reload class.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected AccountProxyInterface $currentUser,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function compress(array $viewsreference, ViewExecutable $view): array {

    // Remove any stored data and existing compression, instead opting for a
    // reload mechanism to load the configuration from source again.
    unset($viewsreference['data']);
    unset($viewsreference['compressed']);

    // Compress the Views Reference Field settings as a JSON String to help
    // avoid 414 errors with the request URI being too long.
    $json = Json::encode($viewsreference);
    return [
      'reload' => UrlHelper::compressQueryParameter($json),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function uncompress(array $viewsreference, ViewExecutable $view): array {
    if (!isset($viewsreference['reload'])) {
      return $viewsreference;
    }

    $entity = FALSE;
    $reload = UrlHelper::uncompressQueryParameter($viewsreference['reload']);
    $reload = Json::decode($reload);
    $entity_storage = $this->entityTypeManager->getStorage($reload['parent_entity_type']);
    if (empty($reload['parent_revision_id'])) {
      $entity = $entity_storage->load($reload['parent_entity_id']);
    }
    else {
      // Load the entity with specific revision. Determine the correct
      // keys to use by the entity type class in case they differ
      // from the usual id and revision_id.
      $entity_keys = $entity_storage->getEntityType()->getKeys();
      $entities = $entity_storage->loadByProperties([
        $entity_keys['id'] => $reload['parent_entity_id'],
        $entity_keys['revision'] => $reload['parent_revision_id'],
      ]);
      if ($entities) {
        $entity = reset($entities);
      }
    }

    // Safely load the data loaded in the field formatter. Step by step
    // ensure that each bit of the recorded entity is available and matches to
    // avoid an unforeseen errors. Double-check access as well in case the
    // user manages to manipulate the compressed query string to result in a
    // different entity ID.
    if (
      $entity instanceof FieldableEntityInterface
      && $entity->hasField($reload['parent_field_name'])
      && $entity->access('view', $this->currentUser)
      && $items = $entity->get($reload['parent_field_name'])
    ) {

      // Retrieve the data that was unset in compression.
      $reload['data'] = [];
      foreach ($items as $delta => $item) {
        if (isset($reload['field_item_delta']) && $reload['field_item_delta'] !== $delta) {
          continue;
        }
        if (!empty($item->getValue()['data'])) {
          $reload['data'] = unserialize($item->getValue()['data'], [
            'allowed_classes' => FALSE,
          ]);
        }
      }
    }
    return $reload;
  }

}
