<?php

namespace Drupal\Tests\viewsreference_extras\FunctionalJavascript;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\views\Views;

/**
 * Tests the views reference extras compression and reload of settings.
 *
 * @group viewsreference_extras
 */
class ViewsReferenceExtrasCompressionReloadTest extends WebDriverTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with permission to bypass access content.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * The node containing the views reference.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'viewsreference',
    'views_ui',
    'field_ui',
    'big_pipe',
    'block',
    'test_views_reference_ajax_history',
    'viewsreference_extras_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalCreateContentType([
      'type' => 'page',
      'name' => 'Basic page',
      'display_submitted' => FALSE,
    ]);
    $this->adminUser = $this->drupalCreateUser([
      'access content',
      'bypass node access',
      'administer nodes',
      'administer content types',
      'administer node fields',
    ]);

    // Create a content item with a views reference. The easiest way to test
    // this is to ensure ajax history is also enabled, so we piggy-back
    // off of that test module.
    $this->node = $this->drupalCreateNode([
      'title' => 'Test page with compression',
      'type' => 'test_ajax_history',
      'field_views_reference_history' => [
        'target_id' => 'test_views_ajax_history',
        'display_id' => 'block_1',
      ],
    ]);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests the Views Reference compression.
   */
  public function testViewsReferenceCompression() {
    // Create sample content items.
    foreach (range(1, 100) as $count) {
      $this->drupalCreateNode([
        'title' => 'Test page for view ' . $count,
      ]);
    }

    // Enable the settings.
    $this->drupalGet('admin/structure/types/manage/test_ajax_history/fields/node.test_ajax_history.field_views_reference_history');
    $this->submitForm([
      'settings[enabled_settings][limit]' => TRUE,
      'settings[enabled_settings][test_compression_reload_setting]' => TRUE,
    ], 'Save settings');
    $this->drupalGet('node/' . $this->node->id() . '/edit');
    $this->click('#edit-field-views-reference-history-0-options');
    $this->submitForm([
      'field_views_reference_history[0][options][limit]' => 20,
      'field_views_reference_history[0][options][test_compression_reload_setting][test_option_2]' => 'test_option_2',
      'field_views_reference_history[0][options][test_compression_reload_setting][test_option_3]' => 'test_option_3',
      'field_views_reference_history[0][options][test_compression_reload_setting][test_option_4]' => 'test_option_4',
    ], 'Save');

    $this->drupalGet('node/' . $this->node->id() . '/edit');
    $this->getSession()->getPage();

    // Test pagination triggers ajax history.
    $this->drupalGet('node/' . $this->node->id());
    $this->assertSession()->pageTextContains('Test page for view 100');
    $pagination_links = $this->getSession()->getPage()->findAll('css', '.pager__item a');
    foreach ($pagination_links as $link) {
      if (str_contains($link->getText(), '2')) {
        $link->click();
        break;
      }
    }
    $this->assertSession()->waitForText('Test page for view 80');
    $this->assertStringContainsString('page=1', $this->getSession()->getCurrentUrl());

    // Next page.
    $pagination_links = $this->getSession()->getPage()->findAll('css', '.pager__item a');
    foreach ($pagination_links as $link) {
      if (str_contains($link->getText(), '3')) {
        $link->click();
        break;
      }
    }
    $this->assertSession()->waitForText('Test page for view 60');
    $this->assertStringContainsString('page=2', $this->getSession()->getCurrentUrl());

    // By default, Views Ajax History will have the viewsreference configuration
    // in the URL on the second click and onwards if the compression is not in
    // place. This ensures the compression has remained in place.
    $this->assertStringContainsString('viewsreference[reload]', $this->getSession()->getCurrentUrl());

    // Ensure that the compressed string, once un-compressed, resulted in the
    // next page of results.
    $this->assertSession()->waitForText('Test page for view 40');
    $parsed_url = UrlHelper::parse($this->getSession()->getCurrentUrl());

    // Check that the uncompressed value is still as expected.
    $expected_data = [
      'title' => NULL,
      'pager' => NULL,
      'offset' => NULL,
      'limit' => '20',
      'header' => NULL,
      'argument' => NULL,
      'test_compression_reload_setting' => [],
    ];

    // Expected configuration should initially only contain what is needed
    // to reload.
    $expected_configuration = [
      'enabled_settings' => [
        'limit' => 'limit',
        'test_compression_reload_setting' => 'test_compression_reload_setting',
      ],
      'parent_entity_type' => 'node',
      'parent_entity_id' => '1',
      'parent_field_name' => 'field_views_reference_history',
    ];
    $json = UrlHelper::uncompressQueryParameter($parsed_url['query']['viewsreference']['reload']);
    $parameters = Json::decode($json);
    $this->assertSame($expected_configuration, $parameters);

    // Let's check what the uncompress would return and make sure that is
    // working correctly.
    // Set the selected 3 settings, leaving the rest empty.
    foreach (range(0, 500) as $number) {
      $expected_data['test_compression_reload_setting']['test_option_' . $number] = 0;
    }
    $expected_data['test_compression_reload_setting']['test_option_2'] = 'test_option_2';
    $expected_data['test_compression_reload_setting']['test_option_3'] = 'test_option_3';
    $expected_data['test_compression_reload_setting']['test_option_4'] = 'test_option_4';
    ksort($expected_data);
    $expected_configuration = [
      'data' => $expected_data,
      'enabled_settings' => [
        'limit' => 'limit',
        'test_compression_reload_setting' => 'test_compression_reload_setting',
      ],
      'parent_entity_type' => 'node',
      'parent_entity_id' => '1',
      'parent_field_name' => 'field_views_reference_history',
    ];
    $view = Views::getView('test_views_ajax_history');
    $view->setDisplay('block_1');
    /** @var \Drupal\viewsreference\ViewsReferenceCompressionInterface $compression */
    $compression = \Drupal::service('viewsreference.compression');
    $parameters = $compression->compress($parameters, $view);
    $result = $compression->uncompress($parameters, $view);

    // We do not care about the order, we just want them to have the same data.
    ksort($result);
    ksort($result['data']);
    ksort($result['data']['test_compression_reload_setting']);
    ksort($expected_configuration);
    ksort($expected_configuration['data']);
    ksort($expected_configuration['data']['test_compression_reload_setting']);
    $this->assertSame($expected_configuration, $result);

    // Check that the other parameters of the view have not been negatively
    // affected: we should now see page 2 in the query parameters.
    $this->assertSame('2', UrlHelper::uncompressQueryParameter($parsed_url['query']['page']));

    // Finally we need to make sure that the Views Reference Field module
    // is still actually respecting the updated configuration returned in the
    // reload. The configured override of 20 per page rather than default 5
    // should still be respected here after reload of settings from the entity.
    $this->assertSession()->elementsCount('css', '.views-row', 20);
  }

}
